-- DEBUG

local debug = false

local function DebugPrint(value)
	if debug then
		print(value)
	end
end

DebugPrint "Starting BuffEnough"

-- Globals
local lastUpdate = 0
local Druid = "Druid"
local Mage = "Mage"
local Priest = "Priest"
local settings
local missingBuff = false
local missingBuffTooltip = ""

--SLASH_


-- Methods


local function SetContains(set, value)
	if set ~= nil then
		for i, entry in ipairs(set) do
			if entry == value then
				return true
			end
		end
	else
		-- We have all the required buffs for a blank set.
		return true
	end

	return false
end

local function CheckPlayerForBuff(spellIDsRequired)
	for i = 1, 40 do
		local name, _, _, _, _, expires, _, _, _, spellID = UnitBuff("player", i)
		
		if name ~= nil then
			if SetContains(spellIDsRequired, spellID) then
				local remains = 0

				if expires > 0 then
					remains = expires - GetTime()
				
					if remains > 0 then				
						return true
					end
				else
					return true
				end
			end
		end
	end

	return false
end

-- Checking Individual Items
-- =========================
-- Required Cast buffs
-- -------------------------
-- Arcane Intellect
-- Mark of the Wild
-- Power Word Fortitude
-- Blessing of Kings
-- Blessing of Might
-- Blessing of Wisdom
--
-- Optional Cast buffs
-- -------------------------
-- Shadow Protection (Priest - Prayer: 27683, Direct: 16874)
-- Divine Spirit (Disc Priests Only - Direct: 16875/27841)
-- Thorns (Druid - Direct 16877)
--
-- Consumables
-- -------------------------
--
--

-- CheckArcaneIntellect
-- Description: Checks whether the player needs and has the highest rank.
local function CheckArcaneIntellect(maxMageLevel)
	local spellIDsRequired;


	if maxMageLevel > 60 then
		spellIDsRequired = {1459}
	elseif maxMageLevel >= 56 then
		spellIDsRequired = {10157,23028}
	elseif maxMageLevel >= 42 then
		spellIDsRequired = {10156}
	elseif maxMageLevel >= 28 then
		spellIDsRequired = {1461}
	elseif maxMageLevel >= 14 then
		spellIDsRequired = {1460}
	elseif maxMageLevel >= 1 then
		spellIDsRequired = {1459}
	end
	
	if spellIDsRequired ~= nil then
		return CheckPlayerForBuff(spellIDsRequired)
	else
		return true
	end
end

-- CheckMarkOfTheWild
-- Description: Checks whether the player needs and has the highest rank of Mark of the Wild.
local function CheckMarkOfTheWild(maxDruidLevel)
	local spellIDsRequired;


	if maxDruidLevel > 60 then
		spellIDsRequired = nil
	elseif maxDruidLevel == 60 then
		spellIDsRequired = {9885,21850}
	elseif maxDruidLevel >= 50 then
		spellIDsRequired = {9884,21849}
	elseif maxDruidLevel >= 40 then
		spellIDsRequired = {8907}
	elseif maxDruidLevel >= 30 then
		spellIDsRequired = {5234}
	elseif maxDruidLevel >= 20 then
		spellIDsRequired = {6756}
	elseif maxDruidLevel >= 11 then
		spellIDsRequired = {5232}
	else
		spellIDsRequired = {1126}
	end
	
	if spellIDsRequired ~= nil then
		return CheckPlayerForBuff(spellIDsRequired)
	else
		return true
	end
end

-- CheckPowerWordFortitude
-- Description: Checks whether the player needs and has the highest rank.
local function CheckPowerWordFortitude(maxPriestLevel)
	local spellIDsRequired;


	if maxPriestLevel > 60 then
		spellIDsRequired = {21562}
	elseif maxPriestLevel >= 56 then
		spellIDsRequired = {10938,21564}
	elseif maxPriestLevel >= 48 then
		spellIDsRequired = {10937,21562}
	elseif maxPriestLevel >= 36 then
		spellIDsRequired = {2791}
	elseif maxPriestLevel >= 24 then
		spellIDsRequired = {1245}
	elseif maxPriestLevel >= 12 then
		spellIDsRequired = {1244}
	elseif maxPriestLevel >= 1 then
		spellIDsRequired = {1243}
	end
	
	if spellIDsRequired ~= nil then
		return CheckPlayerForBuff(spellIDsRequired)
	else
		return true
	end
end


function BuffBox_OnEnter()
  GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
  GameTooltip:SetText(missingBuffTooltip)
  GameTooltip:Show()
end

function BuffBox_OnLeave()
  GameTooltip:Hide()
end

function BuffBox_OnDragStop()
	BuffEnough_Box:StopMovingOrSizing();

	settings.XPos = BuffEnough_Box:GetLeft();
	settings.YPos = BuffEnough_Box:GetBottom();

	DebugPrint("Set Position "..settings.XPos..","..settings.YPos)
end

function BuffBox_OnLoad()
	BuffEnough_Box:RegisterForDrag("LeftButton");
	BuffEnough_Box:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function BuffBox_OnEvent(event)
	if event == "PLAYER_ENTERING_WORLD" then
		DebugPrint("Loading Settings")

		BuffEnoughClassic_Settings = BuffEnoughClassic_Settings  or {}

		settings = BuffEnoughClassic_Settings 

		if settings.XPos then
			DebugPrint("Loading Position "..settings.XPos..","..settings.YPos)

			BuffEnough_Box:ClearAllPoints()
			BuffEnough_Box:SetPoint("BOTTOMLEFT",settings.XPos,settings.YPos)
		end
	end
end


function UpdateDisplay()
	local curTime = GetTime()

	if (lastUpdate + 0.5) < curTime then
		local numGroupMembers = GetNumGroupMembers()

		DebugPrint("Update At "..curTime)

		local maxMageLevel = 0
		local maxDruidLevel = 0
		local maxPriestLevel = 0
		--	local paladinLevels = {}

		local playerLevel = UnitLevel("player")
		local playerClass = UnitClass("player")

		if playerClass  == Druid then
			maxDruidLevel = playerLevel
		elseif playerClass == Mage then
			maxMageLevel = playerLevel
		elseif playerClass == Priest then
			maxPriestLevel = playerLevel
		end


		if IsInRaid() then
			for i = 1, numGroupMembers do
				local unit = "raid"..i

				if UnitExists(unit) then
					local name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i)

					if class == Druid then
						if level > maxDruidLevel then
							maxDruidLevel = level
						end
					elseif class == Mage then
						if level > maxMageLevel then
							maxMageLevel = level
						end
					elseif class == Priest then
						if level > maxPriestLevel then
							maxPriestLevel = level
						end
					end
				end
			end
		elseif IsInGroup() then
			for i = 1, numGroupMembers-1 do
				local unit = "party"..i

				if UnitExists(unit) then
					local name = UnitName(unit)
					local class = UnitClass(unit)
					local level = UnitLevel(unit)

					DebugPrint(name.." "..class.." "..level)

					if class == Druid then
						if level > maxDruidLevel then
							maxDruidLevel = level
						end
					elseif class == Mage then
						if level > maxMageLevel then
							maxMageLevel = level
						end
					elseif class == Priest then
						if level > maxPriestLevel then
							maxPriestLevel = level
						end
					end
				end
			end
		end

		missingBuff = false
		missingBuffTooltip = "Missing Buffs\r\n=========================="

		-- If we have any Druids, make sure we have MOTW.
		if maxDruidLevel > 0 then
			if CheckMarkOfTheWild(maxDruidLevel) == false then
				missingBuff = true
				missingBuffTooltip = missingBuffTooltip.."\r\nMissing Max Level MOTW from a "..maxDruidLevel.." Druid"
			end
		end

		-- If we have any Mages, make sure we have Arcane Intellect.
		if maxMageLevel > 0 then
			if CheckArcaneIntellect(maxMageLevel) == false then
				missingBuff = true
				missingBuffTooltip = missingBuffTooltip.."\r\nMissing Max Level AI from a "..maxMageLevel.." Mage"
			end
		end

				-- If we have any Priests, make sure we have Power Word: Fortitude
		if maxPriestLevel > 0 then
			if CheckPowerWordFortitude(maxPriestLevel) == false then
				missingBuff = true
				missingBuffTooltip = missingBuffTooltip.."\r\nMissing Max Level Fortitude from a "..maxPriestLevel.." Mage"
			end
		end

		lastUpdate = curTime

		if missingBuff then
			--DebugPrint(missingBuffTooltip)

			BuffTexture:SetColorTexture(1.0,0,0)

		else
			--Debug("No Missing Buffs")

			missingBuffTooltip = "No missing buffs!"
			BuffTexture:SetColorTexture(0,0.4,0)
		end
	end
end


-- Main Execution


--buffBox:SetFrameStrata("MEDIUM")
--buffBox:SetWidth(128)
--buffBox:SetHeight(64)
--buffBox.texture = tex
--tex:SetAllPoints()
--tex:SetColorTexture(1.0,0.5,0)
--tex:SetAlpha(0.5)
--buffBox:SetPoint("TOPLEFT", 0, 0)
--buffBox:SetMovable(true)
--buffBox:EnableMouse(true)
--buffBox:RegisterForDrag("LeftButton")
--buffBox:SetScript("OnDragStart", buffBox.StartMoving)
--buffBox:SetScript("OnDragStop", buffBox.StopMovingOrSizing)
--buffBox:SetScript("OnEnter", BuffBox_OnEnter)
--buffBox:SetScript("OnLeave", BuffBox_OnLeave)
--buffBox:Show()
--buffBox:SetScript("OnUpdate", updateDisplay)

DebugPrint("BuffEnough Setup complete")